@@ -36,10 +36,14 @@
 #include <X11/Xlib.h>
 #include <X11/Xatom.h>
 
+// libdconf-qt
+#include "qconf.h"
+
 #include <keyboardmodifiersmonitor.h>
 #include <hotkey.h>
 #include <hotkeymonitor.h>
-#include <dragdropevent.h>
+#include <keymonitor.h>
+#include <debug_p.h>
 
 static const int KEY_HOLD_THRESHOLD = 250;
 
@@ -53,40 +57,44 @@ static const char* SPREAD_DBUS_INTERFACE = "com.canonical.Unity2d.Spread";
 static const char* DASH_DBUS_PROPERTY_ACTIVE = "active";
 static const char* DASH_DBUS_METHOD_ACTIVATE_HOME = "activateHome";
 static const char* SPREAD_DBUS_METHOD_IS_SHOWN = "IsShown";
-static const char* APPLICATIONS_PLACE = "/usr/share/unity/places/applications.place";
-static const char* COMMANDS_PLACE_ENTRY = "Runner";
+static const char* COMMANDS_LENS_ID = "commands.lens";
+static const char* LAUNCHER_DCONF_SCHEMA = "com.canonical.Unity2d.Launcher";
 
 LauncherView::LauncherView(QWidget* parent) :
-    QDeclarativeView(parent),
+    Unity2DDeclarativeView(parent),
     m_superKeyPressed(false), m_superKeyHeld(false)
 {
+    setTransparentBackground(QX11Info::isCompositingManagerRunning());
+
     m_superKeyHoldTimer.setSingleShot(true);
     m_superKeyHoldTimer.setInterval(KEY_HOLD_THRESHOLD);
     connect(&m_superKeyHoldTimer, SIGNAL(timeout()), SLOT(updateSuperKeyHoldState()));
     connect(this, SIGNAL(superKeyTapped()), SLOT(toggleDash()));
-    connect(this, SIGNAL(superKeyHeldChanged(bool)), SLOT(changeKeyboardShortcutsState(bool)));
 
-    m_enableSuperKey.setKey("/desktop/unity-2d/launcher/super_key_enable");
-    connect(&m_enableSuperKey, SIGNAL(valueChanged()), SLOT(updateSuperKeyMonitoring()));
+    m_dconf_launcher = new QConf(LAUNCHER_DCONF_SCHEMA);
+    connect(m_dconf_launcher, SIGNAL(superKeyEnableChanged(bool)), SLOT(updateSuperKeyMonitoring()));
     updateSuperKeyMonitoring();
 
     /* Alt+F1 gives the keyboard focus to the launcher. */
     Hotkey* altF1 = HotkeyMonitor::instance().getHotkeyFor(Qt::Key_F1, Qt::AltModifier);
-    connect(altF1, SIGNAL(pressed()), SLOT(activateWindow()));
+    connect(altF1, SIGNAL(pressed()), SLOT(forceActivateWindow()));
 
-    /* Alt+F2 shows the dash with the commands place entry activated. */
+    /* Alt+F2 shows the dash with the commands lens activated. */
     Hotkey* altF2 = HotkeyMonitor::instance().getHotkeyFor(Qt::Key_F2, Qt::AltModifier);
-    connect(altF2, SIGNAL(pressed()), SLOT(showCommandsPlace()));
+    connect(altF2, SIGNAL(pressed()), SLOT(showCommandsLens()));
 
-    /* Super+s activated the workspaces switcher. */
-    Hotkey* superS = HotkeyMonitor::instance().getHotkeyFor(Qt::Key_S, Qt::MetaModifier);
-    connect(superS, SIGNAL(pressed()), SLOT(showWorkspaceSwitcher()));
+    /* Super+{n} for 0 ≤ n ≤ 9 activates the item with index (n + 9) % 10. */
+    for (Qt::Key key = Qt::Key_0; key <= Qt::Key_9; key = (Qt::Key) (key + 1)) {
+        Hotkey* hotkey = HotkeyMonitor::instance().getHotkeyFor(key, Qt::MetaModifier);
+        connect(hotkey, SIGNAL(pressed()), SLOT(forwardNumericHotkey()));
+        hotkey = HotkeyMonitor::instance().getHotkeyFor(key, Qt::MetaModifier | Qt::ShiftModifier);
+        connect(hotkey, SIGNAL(pressed()), SLOT(forwardNumericHotkey()));
+    }
 }
 
-void
-LauncherView::activateWindow()
+LauncherView::~LauncherView()
 {
-    QDeclarativeView::activateWindow();
+    delete m_dconf_launcher;
 }
 
 void
@@ -107,17 +115,26 @@ void
 LauncherView::updateSuperKeyMonitoring()
 {
     KeyboardModifiersMonitor *modifiersMonitor = KeyboardModifiersMonitor::instance();
+    KeyMonitor *keyMonitor = KeyMonitor::instance();
 
-    QVariant value = m_enableSuperKey.getValue();
+    QVariant value = m_dconf_launcher->property("superKeyEnable");
     if (!value.isValid() || value.toBool() == true) {
         QObject::connect(modifiersMonitor,
                          SIGNAL(keyboardModifiersChanged(Qt::KeyboardModifiers)),
                          this, SLOT(setHotkeysForModifiers(Qt::KeyboardModifiers)));
+        /* Ignore Super presses if another key was pressed simultaneously
+           (i.e. a shortcut). https://bugs.launchpad.net/unity-2d/+bug/801073 */
+        QObject::connect(keyMonitor,
+                         SIGNAL(keyPressed()),
+                         this, SLOT(ignoreSuperPress()));
         setHotkeysForModifiers(modifiersMonitor->keyboardModifiers());
     } else {
         QObject::disconnect(modifiersMonitor,
                             SIGNAL(keyboardModifiersChanged(Qt::KeyboardModifiers)),
                             this, SLOT(setHotkeysForModifiers(Qt::KeyboardModifiers)));
+        QObject::disconnect(keyMonitor,
+                            SIGNAL(keyPressed()),
+                            this, SLOT(ignoreSuperPress()));
         m_superKeyHoldTimer.stop();
         m_superKeyPressed = false;
         if (m_superKeyHeld) {
@@ -137,6 +154,7 @@ LauncherView::setHotkeysForModifiers(Qt::KeyboardModifiers modifiers)
     if (m_superKeyPressed != superKeyPressed) {
         m_superKeyPressed = superKeyPressed;
         if (superKeyPressed) {
+            m_superPressIgnored = false;
             /* If the key is pressed, start up a timer to monitor if it's being held short
                enough to qualify as just a "tap" or as a proper hold */
             m_superKeyHoldTimer.start();
@@ -144,10 +162,12 @@ LauncherView::setHotkeysForModifiers(Qt::KeyboardModifiers modifiers)
             m_superKeyHoldTimer.stop();
 
             /* If the key is released, and was not being held, it means that the user just
-               performed a "tap". Otherwise the user just terminated a hold. */
-            if (!m_superKeyHeld) {
+               performed a "tap". Unless we're told to ignore that tap, that is. */
+            if (!m_superKeyHeld && !m_superPressIgnored) {
                 Q_EMIT superKeyTapped();
-            } else {
+            }
+            /* Otherwise the user just terminated a hold. */
+            else if(m_superKeyHeld){
                 m_superKeyHeld = false;
                 Q_EMIT superKeyHeldChanged(m_superKeyHeld);
             }
@@ -159,51 +179,44 @@ void
 LauncherView::updateSuperKeyHoldState()
 {
     /* If the key was released in the meantime, just do nothing, otherwise
-       consider the key being held. */
-    if (m_superKeyPressed) {
+       consider the key being held, unless we're told to ignore it. */
+    if (m_superKeyPressed && !m_superPressIgnored) {
         m_superKeyHeld = true;
         Q_EMIT superKeyHeldChanged(m_superKeyHeld);
     }
 }
 
 void
-LauncherView::changeKeyboardShortcutsState(bool enabled)
+LauncherView::ignoreSuperPress()
 {
-    /* We are going to connect 10 Hotkeys, but to make things simpler on the QML
-       side we want to have only one signal with the number of the item that needs to
-       be activated in response to the hotkey press.
-       So we connect all of them to a single slot where we emit a single signal with
-       an index based on which Hotkey was the sender. */
-    Qt::Key key = Qt::Key_0;
-    while (key <= Qt::Key_9) {
-        Hotkey *hotkey = HotkeyMonitor::instance().getHotkeyFor(key, Qt::MetaModifier);
-
-        if (enabled) {
-            QObject::connect(hotkey, SIGNAL(pressed()), this, SLOT(forwardHotkey()));
-        } else {
-            QObject::disconnect(hotkey, SIGNAL(pressed()), this, SLOT(forwardHotkey()));
-        }
-        key = (Qt::Key) (key + 1);
-    }
+    /* There was a key pressed, ignore current super tap/hold */
+    m_superPressIgnored = true;
 }
 
 void
-LauncherView::forwardHotkey()
+LauncherView::forwardNumericHotkey()
 {
-    Hotkey *hotkey = qobject_cast<Hotkey*>(sender());
+    Hotkey* hotkey = qobject_cast<Hotkey*>(sender());
     if (hotkey != NULL) {
         /* Shortcuts from 1 to 9 should activate the items with index
-           from 0 to 8. Shortcut for 0 should activate item with index 10.
+           from 1 to 9 (index 0 being the so-called "BFB" or Dash launcher).
+           Shortcut for 0 should activate item with index 10.
            In other words, the indexes are activated in the same order as
            the keys appear on a standard keyboard. */
-        if (hotkey->key() < Qt::Key_0 || hotkey->key() > Qt::Key_9) {
-            return;
-        }
-        int itemIndex = hotkey->key() - Qt::Key_0;
-        itemIndex = (itemIndex == 0) ? 9 : itemIndex - 1;
-
-        if (itemIndex >= 0 && itemIndex <= 10) {
-            Q_EMIT keyboardShortcutPressed(itemIndex);
+        Qt::Key key = hotkey->key();
+        if (key >= Qt::Key_1 && key <= Qt::Key_9) {
+            int index = key - Qt::Key_0;
+            if (hotkey->modifiers() & Qt::ShiftModifier) {
+                Q_EMIT newInstanceShortcutPressed(index);
+            } else {
+                Q_EMIT activateShortcutPressed(index);
+            }
+        } else if (key == Qt::Key_0) {
+            if (hotkey->modifiers() & Qt::ShiftModifier) {
+                Q_EMIT newInstanceShortcutPressed(10);
+            } else {
+                Q_EMIT activateShortcutPressed(10);
+            }
         }
     }
 }
@@ -215,7 +228,7 @@ LauncherView::toggleDash()
 
     QVariant dashActiveResult = dashInterface.property(DASH_DBUS_PROPERTY_ACTIVE);
     if (!dashActiveResult.isValid()) {
-        qWarning() << "Can't read the DBUS Dash property" << DASH_DBUS_PROPERTY_ACTIVE
+        UQ_WARNING << "Can't read the DBUS Dash property" << DASH_DBUS_PROPERTY_ACTIVE
                    << "on" << DASH_DBUS_SERVICE << DASH_DBUS_PATH << DASH_DBUS_INTERFACE;
         return;
     }
@@ -223,7 +236,7 @@ LauncherView::toggleDash()
     bool dashActive = dashActiveResult.toBool();
     if (dashActive) {
         if (!dashInterface.setProperty(DASH_DBUS_PROPERTY_ACTIVE, false)) {
-            qWarning() << "Can't set the DBUS Dash property" << DASH_DBUS_PROPERTY_ACTIVE
+            UQ_WARNING << "Can't set the DBUS Dash property" << DASH_DBUS_PROPERTY_ACTIVE
                        << "on" << DASH_DBUS_SERVICE << DASH_DBUS_PATH << DASH_DBUS_INTERFACE;
         }
     } else {
@@ -250,132 +263,10 @@ LauncherView::toggleDash()
     }
 }
 
-QList<QUrl>
-LauncherView::getEventUrls(DeclarativeDragDropEvent* event)
-{
-    const QMimeData* mimeData = event->mimeData();
-    if (mimeData->hasUrls()) {
-        return mimeData->urls();
-    }
-    else if (mimeData->hasText()) {
-        /* When dragging an URL from firefox’s address bar, it is properly
-           recognized as such by the event. However, the same doesn’t work
-           for chromium: the URL is recognized as plain text.
-           We cope with this unfriendly behaviour by trying to build a URL out
-           of the text. This assumes there’s only one URL. */
-        QString text = mimeData->text();
-        QUrl url(text);
-        if (url.isRelative()) {
-            /* On top of that, chromium sometimes chops off the scheme… */
-            url = QUrl("http://" + text);
-        }
-        if (url.isValid()) {
-            QList<QUrl> urls;
-            urls.append(url);
-            return urls;
-        }
-    }
-    return QList<QUrl>();
-}
-
-void LauncherView::onDragEnter(DeclarativeDragDropEvent* event)
-{
-    Q_FOREACH(QUrl url, getEventUrls(event)) {
-        if ((url.scheme() == "file" && url.path().endsWith(".desktop")) ||
-            url.scheme().startsWith("http")) {
-            event->setAccepted(true);
-            return;
-        }
-    }
-}
-
-void LauncherView::onDrop(DeclarativeDragDropEvent* event)
-{
-    foreach (QUrl url, getEventUrls(event)) {
-        if (url.scheme() == "file" && url.path().endsWith(".desktop")) {
-            emit desktopFileDropped(url.path());
-        }
-        else if (url.scheme().startsWith("http")) {
-            emit webpageUrlDropped(url);
-        }
-    }
-}
-
-/* Calculates both the background color and the glow color of a launcher tile
-   based on the colors in the specified icon (using the same algorithm as Unity).
-   The values are returned as list where the first item is the background color
-   and the second one is the glow color.
-*/
-QList<QVariant>
-LauncherView::getColorsFromIcon(QUrl source, QSize size) const
-{
-    QList<QVariant> colors;
-
-    // FIXME: we should find a way to avoid reloading the icon
-    QImage icon = engine()->imageProvider("icons")->requestImage(source.path().mid(1), &size, size);
-    if (icon.width() == 0 || icon.height() == 0) {
-        qWarning() << "Unable to load icon in getColorsFromIcon from" << source;
-        return colors;
-    }
-
-    long int rtotal = 0, gtotal = 0, btotal = 0;
-    float total = 0.0f;
-
-    for (int y = 0; y < icon.height(); ++y) {
-        for (int x = 0; x < icon.width(); ++x) {
-            QColor color = QColor::fromRgba(icon.pixel(x, y));
-
-            float saturation = (qMax (color.red(), qMax (color.green(), color.blue())) -
-                                qMin (color.red(), qMin (color.green(), color.blue()))) / 255.0f;
-            float relevance = .1 + .9 * (color.alpha() / 255.0f) * saturation;
-
-            rtotal += (unsigned char) (color.red() * relevance);
-            gtotal += (unsigned char) (color.green() * relevance);
-            btotal += (unsigned char) (color.blue() * relevance);
-
-            total += relevance * 255;
-        }
-    }
-
-    QColor hsv = QColor::fromRgbF(rtotal / total, gtotal / total, btotal / total).toHsv();
-
-    /* Background color is the base color with 0.90f HSV value */
-    hsv.setHsvF(hsv.hueF(),
-                (hsv.saturationF() > .15f) ? 0.65f : hsv.saturationF(),
-                0.90f);
-    colors.append(QVariant::fromValue(hsv.toRgb()));
-
-    /* Glow color is the base color with 1.0f HSV value */
-    hsv.setHsvF(hsv.hueF(), hsv.saturationF(), 1.0f);
-    colors.append(QVariant::fromValue(hsv.toRgb()));
-
-    return colors;
-}
-
 void
-LauncherView::showCommandsPlace()
+LauncherView::showCommandsLens()
 {
     QDBusInterface dashInterface(DASH_DBUS_SERVICE, DASH_DBUS_PATH, DASH_DBUS_INTERFACE);
-    dashInterface.asyncCall("activatePlaceEntry",
-                            APPLICATIONS_PLACE, COMMANDS_PLACE_ENTRY, 0);
+    dashInterface.asyncCall("activateLens", COMMANDS_LENS_ID);
 }
 
-void
-LauncherView::showWorkspaceSwitcher()
-{
-    QDBusInterface spreadInterface(SPREAD_DBUS_SERVICE, SPREAD_DBUS_PATH, SPREAD_DBUS_INTERFACE);
-
-    /* Here we only show the spread, if it's hidden.
-       However on Super+s the spread should exit if it's already running.
-       That is done directly in spread/Workspaces.qml because the spread
-       fully grabs the keyboard, so it's the only place where Super+s can
-       be handled while the spread is active */
-    QDBusReply<bool> isShown = spreadInterface.call("IsShown");
-    if (isShown.isValid()) {
-        if (isShown.value() == false) {
-            spreadInterface.asyncCall("ShowAllWorkspaces", QString());
-        }
-    } else {
-        qWarning() << "Failed to get property IsShown on" << SPREAD_DBUS_SERVICE;
-    }
-}
