@@ -22,6 +22,9 @@
 #ifndef PanelManager_H
 #define PanelManager_H
 
+// QConf
+#include <qconf.h>
+
 // Qt
 #include <QObject>
 #include <QList>
@@ -39,9 +42,14 @@ public:
 private:
     Q_DISABLE_COPY(PanelManager)
     QList<Unity2dPanel*> m_panels;
+    QConf m_conf;
+
+    Unity2dPanel* instantiatePanel(int screen);
+    QStringList loadPanelConfiguration() const;
 
 private Q_SLOTS:
     void onScreenCountChanged(int newCount);
+    void onF10Pressed();
 };
 
 #endif // PanelManager_H
