@@ -26,13 +26,13 @@
 
 // unity-2d
 #include <debug_p.h>
+#include <gconnector.h>
 
 // Bamf
 #include <bamf-matcher.h>
 #include <bamf-window.h>
 
 // libwnck
-#undef signals
 extern "C" {
 #define WNCK_I_KNOW_THIS_IS_UNSTABLE
 #include <libwnck/libwnck.h>
@@ -42,10 +42,17 @@ extern "C" {
 #include <QDateTime>
 #include <QApplication>
 #include <QDesktopWidget>
+#include <QPoint>
+
+// X11
+#include <X11/Xlib.h>
+#include <X11/Xatom.h>
+#include <QX11Info>
 
 struct WindowHelperPrivate
 {
     WnckWindow* m_window;
+    GConnector m_connector;
 };
 
 WindowHelper::WindowHelper(QObject* parent)
@@ -97,14 +104,13 @@ void WindowHelper::update()
     uint xid = bamfWindow ? bamfWindow->xid() : 0;
 
     if (d->m_window) {
-        g_signal_handlers_disconnect_by_func(d->m_window, gpointer(stateChangedCB), this);
-        g_signal_handlers_disconnect_by_func(d->m_window, gpointer(nameChangedCB), this);
+        d->m_connector.disconnectAll();
         d->m_window = 0;
     }
     if (xid != 0) {
         d->m_window = wnck_window_get(xid);
-        g_signal_connect(G_OBJECT(d->m_window), "name-changed", G_CALLBACK(nameChangedCB), this);
-        g_signal_connect(G_OBJECT(d->m_window), "state-changed", G_CALLBACK(stateChangedCB), this);
+        d->m_connector.connect(G_OBJECT(d->m_window), "name-changed", G_CALLBACK(nameChangedCB), this);
+        d->m_connector.connect(G_OBJECT(d->m_window), "state-changed", G_CALLBACK(stateChangedCB), this);
     }
     stateChanged();
     nameChanged();
@@ -157,4 +163,27 @@ void WindowHelper::unmaximize()
     wnck_window_unmaximize(d->m_window);
 }
 
+void WindowHelper::drag(const QPoint& pos)
+{
+    // this code IMO should ultimately belong to wnck
+    if (wnck_window_is_maximized(d->m_window)) {
+        XEvent xev;
+        QX11Info info;
+        Atom netMoveResize = XInternAtom(QX11Info::display(), "_NET_WM_MOVERESIZE", false);
+        xev.xclient.type = ClientMessage;
+        xev.xclient.message_type = netMoveResize;
+        xev.xclient.display = QX11Info::display();
+        xev.xclient.window = wnck_window_get_xid(d->m_window);
+        xev.xclient.format = 32;
+        xev.xclient.data.l[0] = pos.x();
+        xev.xclient.data.l[1] = pos.y();
+        xev.xclient.data.l[2] = 8; // _NET_WM_MOVERESIZE_MOVE
+        xev.xclient.data.l[3] = Button1;
+        xev.xclient.data.l[4] = 0;
+        XUngrabPointer(QX11Info::display(), QX11Info::appTime());
+        XSendEvent(QX11Info::display(), QX11Info::appRootWindow(info.screen()), false,
+                   SubstructureRedirectMask | SubstructureNotifyMask, &xev);
+    }
+}
+
 #include "windowhelper.moc"
