@@ -27,6 +27,8 @@
 // Qt
 #include <QObject>
 
+class QPoint;
+
 struct WindowHelperPrivate;
 class WindowHelper : public QObject
 {
@@ -44,6 +46,7 @@ public Q_SLOTS:
     void close();
     void minimize();
     void unmaximize();
+    void drag(const QPoint& pos);
 
 private Q_SLOTS:
     void update();
