@@ -23,19 +23,19 @@
 #define HOMEBUTTONAPPLET_H
 
 // Local
-#include <applet.h>
+#include "homebutton.h"
 
-// Qt
-#include <QToolButton>
+// Unity-2d
+#include <panelapplet.h>
 
 class QDBusInterface;
 class LauncherClient;
 
-class HomeButtonApplet : public Unity2d::Applet
+class HomeButtonApplet : public Unity2d::PanelApplet
 {
 Q_OBJECT
 public:
-    HomeButtonApplet();
+    HomeButtonApplet(Unity2dPanel* panel);
 
 protected:
     void enterEvent(QEvent*);
@@ -47,7 +47,7 @@ private Q_SLOTS:
 
 private:
     Q_DISABLE_COPY(HomeButtonApplet)
-    QToolButton* m_button;
+    HomeButton* m_button;
     QDBusInterface* m_dashInterface;
     LauncherClient* m_launcherClient;
 };
