@@ -22,10 +22,8 @@
 #ifndef LEGACYTRAYAPPLET_H
 #define LEGACYTRAYAPPLET_H
 
-// Local
-#include <applet.h>
-
-// Qt
+// Unity-2d
+#include <panelapplet.h>
 
 namespace SystemTray
 {
@@ -33,11 +31,13 @@ class FdoSelectionManager;
 class Task;
 }
 
-class LegacyTrayApplet : public Unity2d::Applet
+class QConf;
+
+class LegacyTrayApplet : public Unity2d::PanelApplet
 {
 Q_OBJECT
 public:
-    LegacyTrayApplet();
+    LegacyTrayApplet(Unity2dPanel* panel);
     virtual ~LegacyTrayApplet();
 
 private Q_SLOTS:
@@ -48,6 +48,8 @@ private:
     Q_DISABLE_COPY(LegacyTrayApplet)
     
     SystemTray::FdoSelectionManager* m_selectionManager;
+    QConf* m_dconfPanel;
+    QStringList m_whitelist;
 };
 
 #endif /* LEGACYTRAYAPPLET_H */
