@@ -31,61 +31,44 @@
 
 #include <X11/Xlib.h>
 
-#undef signals
-#include <gtk/gtk.h>
-
 // unity-2d
+#include <unity2dapplication.h>
 #include <unity2ddebug.h>
-#include <unity2dtr.h>
 
 #include "dashdeclarativeview.h"
 #include "config.h"
 
 int main(int argc, char *argv[])
 {
-    /* gtk needs to be inited, otherwise we get an assert failure in gdk */
-    gtk_init(&argc, &argv);
-    Unity2dDebug::installHandlers();
+    Unity2dApplication::earlySetup(argc, argv);
+    Unity2dApplication application(argc, argv);
+    application.setApplicationName("Unity 2D Dash");
+    QSet<QString> arguments = QSet<QString>::fromList(QCoreApplication::arguments());
 
-    /* Forcing graphics system to 'raster' instead of the default 'native'
-       which on X11 is 'XRender'.
-       'XRender' defaults to using a TrueColor visual. We do _not_ mimick that
-       behaviour with 'raster' by calling QApplication::setColorSpec because
-       of a bug where some pixmaps become blueish:
-
-       https://bugs.launchpad.net/unity-2d/+bug/689877
-    */
-    QApplication::setGraphicsSystem("raster");
-    QApplication application(argc, argv);
-
-    qmlRegisterType<DashDeclarativeView>("Places", 1, 0, "DashDeclarativeView");
+    qmlRegisterType<DashDeclarativeView>("Unity2d", 1, 0, "DashDeclarativeView");
     DashDeclarativeView view;
+    view.setAccessibleName("Dash");
+    view.setUseOpenGL(arguments.contains("-opengl"));
+
     if (!view.connectToBus()) {
         qCritical() << "Another instance of the Dash already exists. Quitting.";
         return -1;
     }
 
-    /* Configure translations */
-    Unity2dTr::init("unity-2d", INSTALL_PREFIX "/share/locale");
-    Unity2dTr::qmlInit(view.rootContext());
-
     view.engine()->addImportPath(unity2dImportPath());
     /* Note: baseUrl seems to be picky: if it does not end with a slash,
        setSource() will fail */
     view.engine()->setBaseUrl(QUrl::fromLocalFile(unity2dDirectory() + "/places/"));
 
     if (!isRunningInstalled()) {
-        /* Place.qml imports UnityApplications, which is part of the launcher
-           component… */
-        view.engine()->addImportPath(unity2dDirectory() + "/launcher/");
         /* Place.qml imports Unity2d */
         view.engine()->addImportPath(unity2dDirectory() + "/libunity-2d-private/");
     }
 
     /* Load the QML UI, focus and show the window */
     view.setResizeMode(QDeclarativeView::SizeRootObjectToView);
+    view.rootContext()->setContextProperty("declarativeView", &view);
     view.rootContext()->setContextProperty("dashView", &view);
-    view.rootContext()->setContextProperty("engineBaseUrl", view.engine()->baseUrl().toLocalFile());
     view.setSource(QUrl("./dash.qml"));
 
     /* When spawned via DBus activation, the current working directory is
